//   Copyright 2022 Esri
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

package com.example.app;

import com.esri.arcgisruntime.ArcGISRuntimeEnvironment;
import com.esri.arcgisruntime.arcgisservices.LabelDefinition;
import com.esri.arcgisruntime.data.ServiceFeatureTable;
import com.esri.arcgisruntime.layers.FeatureLayer;
import com.esri.arcgisruntime.mapping.ArcGISMap;
import com.esri.arcgisruntime.mapping.BasemapStyle;
import com.esri.arcgisruntime.mapping.Viewpoint;
import com.esri.arcgisruntime.mapping.labeling.ArcadeLabelExpression;
import com.esri.arcgisruntime.mapping.view.MapView;
import com.esri.arcgisruntime.symbology.ClassBreaksRenderer;
import com.esri.arcgisruntime.symbology.ClassBreaksRenderer.ClassBreak;
import com.esri.arcgisruntime.symbology.PictureMarkerSymbol;
import com.esri.arcgisruntime.symbology.SimpleFillSymbol;
import com.esri.arcgisruntime.symbology.SimpleLineSymbol;
import com.esri.arcgisruntime.symbology.SimpleRenderer;
import com.esri.arcgisruntime.symbology.TextSymbol;
import com.esri.arcgisruntime.symbology.UniqueValueRenderer;
import com.esri.arcgisruntime.symbology.UniqueValueRenderer.UniqueValue;

import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

import java.util.ArrayList;
import java.util.List;

public class App extends Application {

  private MapView mapView;

  private ArcGISMap map =new ArcGISMap(BasemapStyle.ARCGIS_TOPOGRAPHIC);

  // colors for symbols
  private static final Color RED = Color.RED;
  private static final Color GREEN = Color.GREEN;
  private static final Color BLUE = Color.BLUE;
  private static final Color PURPLE = Color.PURPLE;
  private static final Color WHITE = Color.WHITE;

  // Application constants used to connect to data and resources.
  private final String parksAndOpenSpaces =
      "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Parks_and_Open_Space/FeatureServer/0";
  private final String trails =
      "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trails/FeatureServer/0";
  private final String trailheads =
      "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trailheads/FeatureServer/0";
  private final String trailheadImage = "https://static.arcgis.com/images/Symbols/NPS/npsPictograph_0231b.png";


  public static void main(String[] args) {
    Application.launch(args);
  }

  @Override
  public void start(Stage stage) {

    // set the title and size of the stage and show it
    stage.setTitle("Style a feature layer");

    stage.setWidth(800);
    stage.setHeight(700);
    stage.show();

    // create a JavaFX scene with a stack pane as the root node, and add it to the scene
    StackPane stackPane = new StackPane();
    Scene scene = new Scene(stackPane);
    stage.setScene(scene);

    // Note: it is not best practice to store API keys in source code.
    // The API key is referenced here for the convenience of this tutorial.
    String yourApiKey = "YOUR_API_KEY";
    ArcGISRuntimeEnvironment.setApiKey(yourApiKey);

    // create a map view to display the map and add it to the stack pane
    mapView = new MapView();
    stackPane.getChildren().add(mapView);

    map = new ArcGISMap(BasemapStyle.ARCGIS_TOPOGRAPHIC);

    // set the map on the map view
    mapView.setMap(map);

    mapView.setViewpoint(new Viewpoint(34.02700, -118.80543, 72000.0));

    addOpenSpaceLayer();

    addTrailsLayer();

    addBikeOnlyTrailsLayer();

    addNoBikesTrailsLayer();

    addTrailHeadsLayer();

  }

  /**
   * Stops and releases all resources used in application.
   */
  @Override
  public void stop() {
    if (mapView != null) {
      mapView.dispose();
    }
  }

  private FeatureLayer addFeatureLayer(String featureServiceUri){
    // Create a service feature table from a Uri.
    ServiceFeatureTable serviceFeatureTable = new ServiceFeatureTable(featureServiceUri);
    // Create a feature layer from the service feature table.
    FeatureLayer featureLayer = new FeatureLayer(serviceFeatureTable);
    // Add the feature layer to the map's list of operational layers.
    map.getOperationalLayers().add(featureLayer);
    // return the feature layer
    return featureLayer;
  }

  private void addOpenSpaceLayer() {
    // Create a parks and open spaces feature layer and add it to the map view.
    FeatureLayer featureLayer = addFeatureLayer(parksAndOpenSpaces);

    // Create fill symbols.
    SimpleFillSymbol purpleFillSymbol =
        new SimpleFillSymbol(SimpleFillSymbol.Style.SOLID, PURPLE, null);
    SimpleFillSymbol greenFillSymbol =
        new SimpleFillSymbol(SimpleFillSymbol.Style.SOLID, GREEN, null);
    SimpleFillSymbol blueFillSymbol =
        new SimpleFillSymbol(SimpleFillSymbol.Style.SOLID, BLUE, null);
    SimpleFillSymbol redFillSymbol =
        new SimpleFillSymbol(SimpleFillSymbol.Style.SOLID, RED, null);

    // Create a unique value for natural areas, regional open spaces, local parks, and regional recreation parks.
    List<Object> naturalAreasValue = new ArrayList<>();
    naturalAreasValue.add("Natural Areas");
    UniqueValue naturalAreas =
        new UniqueValue("Natural Areas", "Natural Areas", purpleFillSymbol, naturalAreasValue);

    List<Object> regionalOpenSpaceValue = new ArrayList<>();
    naturalAreasValue.add("Regional Open Space");
    UniqueValue regionalOpenSpace =
        new UniqueValue("Regional Open Space", "Regional Open Space", greenFillSymbol, regionalOpenSpaceValue);

    List<Object> localParkValue = new ArrayList<>();
    localParkValue.add("Local Park");
    UniqueValue localPark =
        new UniqueValue("Local Park", "Local Park", blueFillSymbol, localParkValue);

    List<Object> regionalRecreationParkValue = new ArrayList<>();
    regionalRecreationParkValue.add("Regional Recreation Park");
    UniqueValue regionalRecreationPark =
        new UniqueValue("Regional Recreation Park", "Regional Recreation Park", redFillSymbol, regionalRecreationParkValue);

    List<UniqueValue> uniqueValuesList = new ArrayList<>();
    uniqueValuesList.add(naturalAreas);
    uniqueValuesList.add(regionalOpenSpace);
    uniqueValuesList.add(localPark);
    uniqueValuesList.add(regionalRecreationPark);

    // Create a unique value renderer and set it on the parks and open spaces feature layer.
    // fieldNames is a List, but in this tutorial there is only one item in the list
    List<String> fieldNames = new ArrayList<>();
    fieldNames.add("TYPE");
    UniqueValueRenderer openSpacesUniqueValueRenderer =
        new UniqueValueRenderer(fieldNames, uniqueValuesList, "Open Spaces", null);
    featureLayer.setRenderer(openSpacesUniqueValueRenderer);

    // Set the layer opacity to semi-transparent.
    featureLayer.setOpacity(0.2f);
  }

  private void addTrailsLayer() {
    // Create a trails feature layer and add it to the map view.
    FeatureLayer featureLayer = addFeatureLayer(trails);

    // Create simple line symbols.
    SimpleLineSymbol firstClassSymbol =
        new SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, PURPLE, 3.0f);
    SimpleLineSymbol secondClassSymbol =
        new SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, PURPLE, 4.0f);
    SimpleLineSymbol thirdClassSymbol =
        new SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, PURPLE, 5.0f);
    SimpleLineSymbol fourthClassSymbol =
        new SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, PURPLE, 6.0f);
    SimpleLineSymbol fifthClassSymbol =
        new SimpleLineSymbol(SimpleLineSymbol.Style.SOLID, PURPLE, 7.0f);

    // Create five class breaks.
    ClassBreak firstClassBreak =
        new ClassBreak("Under 500", "0 - 500", 0.0, 500.0, firstClassSymbol);
    ClassBreaksRenderer.ClassBreak secondClassBreak =
        new  ClassBreak("501 to 1000", "501 - 1000", 501.0, 1000.0, secondClassSymbol);
    ClassBreak thirdClassBreak =
        new  ClassBreak("1001 to 1500", "1001 - 1500", 1001.0, 1500.0, thirdClassSymbol);
    ClassBreak fourthClassBreak =
        new  ClassBreak("1501 to 2000", "1501 - 2000", 1501.0, 2000.0, fourthClassSymbol);
    ClassBreak fifthClassBreak =
        new ClassBreak("2001 to 2300", "2001 - 2300", 2001.0, 2300.0, fifthClassSymbol);

    List<ClassBreak> elevationBreaks = new ArrayList<>();
    elevationBreaks.add(firstClassBreak);
    elevationBreaks.add(secondClassBreak);
    elevationBreaks.add(thirdClassBreak);
    elevationBreaks.add(fourthClassBreak);
    elevationBreaks.add(fifthClassBreak);

    // Create a class breaks renderer and set it on the feature layer.
    ClassBreaksRenderer elevationClassBreaksRenderer =
        new ClassBreaksRenderer("ELEV_GAIN", elevationBreaks);
    featureLayer.setRenderer(elevationClassBreaksRenderer);

    // Set the layer opacity to semi-transparent.
    featureLayer.setOpacity(0.75f);

  }

  private void addBikeOnlyTrailsLayer() {
    // Create a trails feature layer and add it to the map view.
    FeatureLayer featureLayer = addFeatureLayer(trails);

    // Write a definition expression to filter for trails that permit the use of bikes.
    featureLayer.setDefinitionExpression("USE_BIKE = 'Yes'");

    // Create a simple renderer and set it on the feature layer.
    SimpleLineSymbol bikeTrailSymbol = new SimpleLineSymbol(SimpleLineSymbol.Style.DOT, BLUE, 2.0f);
    SimpleRenderer bikeTrailRenderer = new SimpleRenderer(bikeTrailSymbol);
    featureLayer.setRenderer(bikeTrailRenderer);

  }

  private void addNoBikesTrailsLayer() {
    // Create a trails feature layer and add it to the map view.
    FeatureLayer featureLayer = addFeatureLayer(trails);

    // Write a definition expression to filter for trails that don't permit the use of bikes.
    featureLayer.setDefinitionExpression("USE_BIKE = 'No'");

    // Create a simple renderer and set it on the feature layer.
    SimpleLineSymbol noBikeTrailSymbol =
        new SimpleLineSymbol(SimpleLineSymbol.Style.DOT, RED, 2.0f);
    SimpleRenderer noBikeTrailRenderer =
        new SimpleRenderer(noBikeTrailSymbol);
    featureLayer.setRenderer(noBikeTrailRenderer);
  }

  private LabelDefinition makeLabelDefinition(String labelAttribute){
    // Create a new text symbol.
    TextSymbol labelTextSymbol  = new TextSymbol();
    labelTextSymbol.setColor(WHITE);
    labelTextSymbol.setSize(12.0f);
    labelTextSymbol.setHaloColor(RED);
    labelTextSymbol.setHaloWidth(1.0f);
    labelTextSymbol.setFontFamily("Arial");
    labelTextSymbol.setFontStyle(TextSymbol.FontStyle.ITALIC);
    labelTextSymbol.setFontWeight(TextSymbol.FontWeight.NORMAL);

    // Create a new Arcade label expression based on the field name.
    ArcadeLabelExpression labelExpression =
        new ArcadeLabelExpression("$feature." + labelAttribute);

    // Create and return the label definition.
    return new LabelDefinition(labelExpression, labelTextSymbol);

  }

  private void addTrailHeadsLayer() {

    // Create a trailheads feature layer and add it to the map view.
    FeatureLayer featureLayer = addFeatureLayer(trailheads);

    // Create a new picture marker symbol that uses the trailhead image.
    PictureMarkerSymbol pictureMarkerSymbol = new PictureMarkerSymbol(trailheadImage);
    pictureMarkerSymbol.setHeight(18.0f);
    pictureMarkerSymbol.setWidth(18.0f);


    // Create a new simple renderer based on the picture marker symbol.
    SimpleRenderer simpleRenderer = new SimpleRenderer(pictureMarkerSymbol);

    // Set the feature layer's renderer, and enable labels.
    featureLayer.setRenderer(simpleRenderer);
    featureLayer.setLabelsEnabled(true);

    // Create the label definition.
    LabelDefinition trailHeadsDefinition = makeLabelDefinition("TRL_NAME");

    // Add the label definition to the layer's list of label definitions.
    featureLayer.getLabelDefinitions().add(trailHeadsDefinition);

  }

}

